#!usr/bin/env python3

import os
from os import path
from fnmatch import fnmatch

upgrading_tagdir = ".ota_upgrading_tagfiles"

upgrading_backup_began_tagfile = "backup_began"
upgrading_backup_failed_tagfile = "backup_failed"
upgrading_rollback_tagfile = "rollback"

pattern_backup = "*.backup"
pattern_to_be_delete = "*.backup.delete"


# assert this file is in gateway/script/
def get_gateway_root():
    """
    assert this file is placed in gateway/script
    :rtype  string
    :return string of
    """
    my_path = path.dirname(path.abspath(__file__))
    root = path.abspath(path.join(my_path, os.pardir))

    return root


def upgrading_tag_dir():
    return path.join(get_gateway_root(), upgrading_tagdir)


def was_upgrading():
    if path.exists(path.join(get_gateway_root(), upgrading_tagdir)):
        return True
    return False


def is_backup_file(filename):
    fn, ext = path.splitext(filename)
    return ext == ".backup"


def is_to_be_delete_file(filename):
    fn, ext = path.splitext(filename)
    fn_origin, ext_backup = path.splitext(fn)
    return ext == ".delete" and ext_backup == ".backup"


def recover_from_backup_began():
    filelist = os.listdir(get_gateway_root())
    tmpfiles = [f for f in filelist if fnmatch(f, "*.tmp")]

    for tmpfile in tmpfiles:
        os.remove(path.join(get_gateway_root(), tmpfile))

    # if any .tmp file exists, delete them
    # if any .backup.delete file exists, rename matching .backup file to *
    #       then rename *.backup.delete to *.backup
    to_be_delete_files = [f for f in filelist if fnmatch(f, pattern_to_be_delete)]
    backup_files = [f for f in filelist if fnmatch(f, pattern_backup)]

    for to_be_delete_file in to_be_delete_files:
        fn, fn_ext = path.splitext(to_be_delete_file)
        if fn in backup_files:
            fn_origin, fn_ext = path.splitext(fn)
            os.rename(fn, fn_origin)
            # rename *.backup -> * done.

    for to_be_delete_file in to_be_delete_files:
        fn, fn_ext = path.splitext(to_be_delete_file)
        os.rename(to_be_delete_file, fn)
        # rename *.backup.delete -> *.backup done


def do_recover_stuff():
    if path.exists(path.join(upgrading_tag_dir(), upgrading_backup_began_tagfile)) \
            or path.exists(path.join(upgrading_tagdir(), upgrading_backup_failed_tagfile)):
        recover_from_backup_began()
        os.remove(path.join(upgrading_tag_dir(), upgrading_backup_began_tagfile))

    else:
        recover_from_upgrading()
        os.remove(upgrading_tag_dir())


def recover_from_upgrade_backup():
    pass


def get_delete_list():
    filelist = os.listdir(get_gateway_root())
    return [path.join(get_gateway_root(), f) for f in filelist if fnmatch(f, pattern_to_be_delete)]


def do_file_rollback_from_upgrading_power_loss():
    # 目前状态：备份已完成，所有解压缩的文件位于 *.tmp，原备份文件夹位于 *.backup.delete，原文件位于 *.backup
    # 恢复：删除对应的 *、*.tmp，然后恢复 *.backup -> *，再恢复 *.backup.delete -> *.backup
    # 以上顺序不能颠倒。
    file_list = get_delete_list()
    tobe_delete_file_list = [f for f in file_list if is_to_be_delete_file(f)]

    # get files list that need to be rename to origin
    backed_up_list = set()
    origin_list = set()
    for fn in tobe_delete_file_list:
        fn_b, ext = path.splitext(fn)
        if is_backup_file(fn_b):
            backed_up_list.add(fn_b)
            fn_o, ext = path.splitext(fn_b)
            origin_list.add(fn_o)

    # delete all *
    for fn in origin_list:
        if path.exists(fn):
            os.remove(fn)

    # rename *.backup -> *
    for fn in backed_up_list:
        if path.exists(fn):
            fn_o, ext = path.splitext(fn)
            os.rename(fn, fn_o)

    # rename *.backup -> *
    for fn in tobe_delete_file_list:
        fn_b, ext = path.splitext(fn)
        os.rename(fn, fn_b)


def recover_from_upgrading():
    filelist = os.listdir(upgrading_tag_dir())
    upgrading_step_files = sorted(
        [path.join(upgrading_tag_dir(), f) for f in filelist if fnmatch(f, "*step_?")],
        key = str.lower)
    if len(upgrading_step_files) == 0:
        recover_from_backup_began()  # 在删除 .backup_began 之后断电

    elif len(upgrading_step_files) == 1:
        # 在某个阶段进行中断电
        step = int(upgrading_step_files[0][-1])

        if step == 0 or step == 1 or step == 2 or step == 3:
            do_file_rollback_from_upgrading_power_loss()
            os.remove(upgrading_step_files[0])

        elif step == 4:
            pass

    elif len(upgrading_step_files) == 2:
        former = int(upgrading_step_files[0][-1])
        latter = int(upgrading_step_files[1][-1])
        if former == 3 and latter == 4:
            os.remove(upgrading_step_files[0])
            # no difference to single file step_4 now.
        else:
            do_file_rollback_from_upgrading_power_loss()

    else:
        pass  # something must be wrong


def main():
    if was_upgrading():
        do_recover_stuff()


if __name__ == '__main__':
    main()
