#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import dbus
import dbus.service
import dbus.proxies
import dbus.bus

iot_bus_file = "/tmp/var/run/mbusd/mbusd_socket"
sec_wkn = 'iot.gateway.security'
sec_iface = 'iot.gateway.security.sst'


def gw_rule_name2path(name):
    # type: (str)->str
    return '/' + name.replace('.', '/')


def gw_rule_generate_key(pk, dn):
    return pk+"_"+dn


def main(*argv):
    parser = argparse.ArgumentParser()
    parser.add_argument("WellKnownName", help="Well known name to be used.", type=str)
    parser.add_argument("-f", "--file", help="Files to be imported", type=str, nargs='+')
    parser.add_argument("Key", help="Key to store.", type=str, nargs="?")
    parser.add_argument("Value", help="Value of key.", type=str, nargs="?")
    args = parser.parse_args()

    importer_wkn = args.WellKnownName
    file_list = args.file

    if file_list is None and (args.Key is None or len(args.Key) == 0) and args.Value is None:
        print("Neither file nor key-value pair was given.")
        parser.print_help()
        parser.exit()

    try:
        me = dbus.bus.BusConnection('unix:path='+iot_bus_file)
        me.request_name(importer_wkn)
    except dbus.exceptions.DBusException as ex:
        print("{}:\n\t{}".format(ex.get_dbus_name(), ex.get_dbus_message()))
        return

    sec_module = me.get_object(sec_wkn, gw_rule_name2path(sec_iface), introspect=False)
    secGet = sec_module.get_dbus_method('secGet', dbus_interface=sec_iface)
    secStore = sec_module.get_dbus_method('secStore', dbus_interface=sec_iface)

    if file_list:
        for _f in file_list:
            with open(_f) as f:
                for l in f.readlines():
                    pk, dn, sk = l.split()
                    secStore(importer_wkn, gw_rule_generate_key(pk, dn), sk)
                    ret = secGet(importer_wkn, gw_rule_generate_key(pk, dn))
                    print(ret)
                    assert ret[1] == sk
    else:
        if args.Key and len(args.Key) and args.Value:
            secStore(importer_wkn, args.Key, args.Value)
            ret = secGet(importer_wkn, args.Key)
            print(ret)
            assert ret[1] == args.Value
        else:
            print(args.Key)
            print(args.Value)
            print("Illegal key-value pair.")


if __name__ == '__main__':
    import sys
    main(*sys.argv)
